/*
** Copyright (c) 2012, David C. M. Ostapchuk (d.ostapchuk@uwinnipeg.ca)
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**
** 1. Redistributions of source code must retain the above copyright notice, this
**    list of conditions and the following disclaimer.
**
** 2. Redistributions in binary form must reproduce the above copyright notice,
**    this list of conditions and the following disclaimer in the documentation
**    and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
** SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
** CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
** OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef EVENTSEQUENCE_HH
#define EVENTSEQUENCE_HH

#include <istream>
#include <ostream>
#include <vector>

/*
** Represents a comparison between two DNA sequences of N sites. The vector
** events stores the location of the variant sites ("events").
*/
typedef struct {
	unsigned int N;
	std::vector<unsigned int> events;  // these should be sorted in ascending order
} eventSequence;


/*
** std::ostream& operator<<( std::ostream &outputStream, const eventSequence &sequence );
** 
** Inserts a string representation of sequence into outputStream.
** 
** The string has the following format:
** 
** N:e1,e2,e3,...
** 
** where
**     N is the total number of sites in the sequence
**     e1, e2, e3, ... are the sites of the events
*/
std::ostream& operator<<( std::ostream &outputStream, const eventSequence &sequence );


/*
** std::istream& operator>>( std::istream &inputStream, eventSequence &sequence );
** 
** Attempts to extract a string representation of sequence from inputStream.
** 
** The string is expected to be in the following format:
** 
** N:e1,e2,e3,...
** 
** where
**     N is the total number of sites in the sequence
**     e1, e2, e3, ... are the sites of the events
*/
std::istream& operator>>( std::istream &inputStream, eventSequence &sequence );

#endif  // EVENTSEQUENCE_HH
