/*
** Copyright (c) 2012, David C. M. Ostapchuk (d.ostapchuk@uwinnipeg.ca)
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**
** 1. Redistributions of source code must retain the above copyright notice, this
**    list of conditions and the following disclaimer.
**
** 2. Redistributions in binary form must reproduce the above copyright notice,
**    this list of conditions and the following disclaimer in the documentation
**    and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
** SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
** CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
** OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef IO_TEMPLATES_CPP
#define IO_TEMPLATES_CPP

#include "eventSequence.hh"

#include <vector>
#include <iostream>
#include <fstream>
#include <string>


/*
** template<class T>
** void printVector( const std::vector<T> &v, std::ostream &stream, const char delimiter=',', const bool printDelimiter=true );
**
** DESCRIPTION:
**
** Prints a vector of values to stream. If printDelimiter is true, each element
** is separated by delimiter. A newline character is not printed after the
** contents of the vector.
**
** By default, the delimiter ',' is printed between elements.
**
** PARAMETERS:
**     const std::vector<T> &v      the vector to print
**     std::ostream &stream         the output stream where the vector is printed
**     const char delimiter         the delimiter to print (default is ',')
**     const bool printDelimiter    whether to print the delimiter or not (default is true)
*/
template<class T>
void printVector( const std::vector<T> &v, std::ostream &stream, const char delimiter, const bool printDelimiter ) {
	if( v.size() > 0 ) {
		stream << v[0];

		if( v.size() > 1 ) {
			for( unsigned int i = 1; i < v.size(); i++ ) {
				if( printDelimiter ) {
					stream << delimiter;
				}
				stream << v[i];
			}
		}
	}
}


/*
** template<class T>
** void printVector( const std::vector< std::vector<T> > &v, std::ostream &stream, const char delimiter=',', const bool printDelimiter=true );
**
** DESCRIPTION:
**
** Prints a vector of vectors to stream. If printDelimiter is true, each element
** is separated by delimiter. A newline character is printed after the contents
** of each vector.
**
** By default, the delimiter ',' is printed between elements.
**
** PARAMETERS:
**     const std::vector< std::vector<T> > &v      the vector to print
**     std::ostream &stream                        the output stream where the vector is printed
**     const char delimiter                        the delimiter to print (default is ',')
**     const bool printDelimiter                   whether to print the delimiter or not (default is true)
*/
template<class T>
void printVector( const std::vector< std::vector<T> > v, std::ostream &stream, const char delimiter, const bool printDelimiter ) {
	for( unsigned int i = 0; i < v.size(); i++ ) {
		printVector( v[i], stream, delimiter, printDelimiter );
		stream << "\n";
	}
}

#endif  // IO_TEMPLATES_CPP
