/*
** Copyright (c) 2012, David C. M. Ostapchuk (d.ostapchuk@uwinnipeg.ca)
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**
** 1. Redistributions of source code must retain the above copyright notice, this
**    list of conditions and the following disclaimer.
**
** 2. Redistributions in binary form must reproduce the above copyright notice,
**    this list of conditions and the following disclaimer in the documentation
**    and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
** SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
** CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
** OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef RANDOM_HH
#define RANDOM_HH

#include <vector>


/*
** void seed_drand48( void ) {
**
** DESCRIPTION:
**
** Seeds drand48 using random numbers generated by rand, which is seeded by the
** current time.
*/
void seed_drand48( void );


/*
** unsigned int randomUnsignedInt( const unsigned int lowLimit, const unsigned int highLimit );
**
** DESCRIPTION:
**
** Returns a random unsigned integer in the interval [lowLimit, highLimit].
**
** The random numbers are generated using drand48 to be consistent with:
**
** Tang, H. and Lewonton, R. C. Locating Regions of Differential Variability in
** DNA and Protein Sequences. Genetics 153(1):485-495, September 1999.
** PMID:10471728, PMCID:PMC1460758, WOS:000082421600035.
**
** PARAMETERS:
**     const unsigned int lowLimit     lowest random number possible
**     const unsigned int highLimit    highest random number possible
**
** RETURNS:
**     A random unsigned integer in the interval [lowLimit, highLimit].
*/
unsigned int randomUnsignedInt( const unsigned int lowLimit, const unsigned int highLimit );


/*
** std::vector<unsigned int> generateUniqueRandomUnsignedInts( const unsigned int a, const unsigned int b, const unsigned int n );
**
** DESCRIPTION:
**
** Returns a vector containing n unique randomly chosen numbers from the interval [a, b].
**
** If n > |b - a| + 1 (i.e., you asked for more numbers than are possible), the
** vector will only have |b - a| + 1 elements.
**
** PARAMETERS:
**     const unsigned int a    lowest possible random number
**     const unsigned int b    highest possible random number
**     const unsigned int n    the number of unique random numbers to generate
**
**
** RETURNS:
**     A vector containing n unique randomly chosen numbers from the interval [a, b].
*/
std::vector<unsigned int> generateUniqueRandomUnsignedInts( const unsigned int a, const unsigned int b, const unsigned int n );


#endif  // RANDOM_HH
